/*-----------------------------------------------------------------
/					 Fichier  CRound.cpp						
/------------------------------------------------------------------
Role: implmentation de la classe CRound
-------------------------------------------------------------------
Versions: (depuis regroupement du 22/12/02)
-------------------------------------------------------------------
23/12/02: regroupement des modifs de Fran et Guillaume
	Appel des fonctions static de chargement d'images de bombes
	et de flammes dans le constructeur, et de destruction dans le 
	destructeur.
02/01/03: Francois - ajout de la musique et de la donnee membre _maMusique
	ainsi que de la gestion de la manette
4/01/03: (G) Modif du constructeur: passage du nombre de parties.
	Cration de l'affichage des scores en haut de l'cran.
	Cration de l'affichage du temps restant.
	TODO: liberer la mm. des images de score
9/01/03: decouverte d'un bug incomprehensible en mode 'release' - au
	chargement du niveau, il manque aleatoirement un avatar (?!?)
	=> le remplacement de atoi(c) par (int)c-48 dans chargerNiveau() 
	semble corriger le pb
12/01/03: Francois - ajout du chargement des images a partir de ce
	qui est indique dans le fichier de niveau 
	+ ecran et images sont maintenant des donnees membres de CRound
14/01/03: Decouverte de la cause de l'absence de fondu en mode Windows
	fullscreen: il faut baisser les parametre d'acclration graphique
	(voir manuel utilisateur)
16/01/03 Francois - ajout du parametre ecran a CBombe::staticInit() 
------------------------------------------------------------------*/


#include "CRound.h"
#include "CBombe.h"
#include <SDL/SDL_ttf.h>


#define DUREE_ROUND 120		//duree d'un round, en secondes
#define REP_NIVEAUX "niveaux/"

//-----------------Constructeur--------------------//
CRound::CRound(CConfiguration & config, int * tabPartie, SDL_Surface * ecran)
:_nbJoueurs(config._nbJoueurs),
 //_gestPad(0),	// 0 = gamepad par defaut
 _maMusique("musique1.mp3"),
 _ecran(ecran) 
{
	string temp(REP_NIVEAUX);
	temp.append(config._nomTerrain.c_str());
	
	_nomNiveau = temp;
	_partie = tabPartie;

	ifstream fichier(_nomNiveau.c_str() ,ios::in);
	string sTexturePath1, sTexturePath2, sTexturePath3, sImgFond, sImgDessus;
	fichier >> sTexturePath1;
	fichier >> sTexturePath2;
	fichier >> sTexturePath3;
	fichier >> sImgFond;
	fichier >> sImgDessus;

	_imageFond =  ImageLoad(sImgFond.c_str());
	_imageDessus = ImageLoad(sImgDessus.c_str());

	SDL_SetColorKey(_imageDessus, SDL_SRCCOLORKEY, SDL_MapRGB(ecran->format, 0, 0, 255));
	
	// On se positionne dans le fichier a l'endroit des donnes 
	// requises par la fonction chargerNiveau()
	char t = fichier.get();
	if ( t == '\r')			// cas de fichiers sous Linux (sacre Linux !)
		t=fichier.get();
	
	
	CFlamme::staticInit();
	_vectAvatar.reserve(config._nbJoueurs);
	CCase::staticInit(this,sTexturePath1, sTexturePath2, sTexturePath3);
	CAvatar::staticInit();
	CBombe::staticInit(this, ecran);
	CBonus::staticInit();
	CGestionnaireGamepad::staticInit();

	for (int i = 0; i<_nbJoueurs; i++)
	{
		// cas d'une manette
		if ( config._configCommandes[0][i] == 1)
			_vectAvatar.push_back(new CAvatar(1,2000,1000,new CGestionnaireGamepad(config._configCommandes[1][i]),this,i,ecran));

		else	
			// cas d'une config clavier
			_vectAvatar.push_back(new CAvatar(1,2000,1000,new CGestionnaireClavier(config._configCommandes[1][i]),this,i,ecran));
	}
	
	chargerNiveau(&fichier);
}

//-----------------Destructeur--------------------//
CRound::~CRound()
{
	CBombe::staticDestruct();
	CAvatar::staticDestruct();
	CCase::staticDestruct();
	CFlamme::staticDestruct();
	CBonus::staticDestruct();

	for (int i = 0; i<_nbJoueurs; i++)
		delete _vectAvatar[i];
	
	for (int y=0; y<NB_CASES_H; y++)
			for (int x=0; x<NB_CASES_L; x++)
				delete _tabCases[x][y];

	SDL_FreeSurface(_imageFond);
	SDL_FreeSurface(_imageDessus);

	_maMusique.liberer();
	_vectADetruire.clear();
}




/* Les emplacements des joueurs sont numerotes de 0 a 9
 * => un niveau peut accueuillir un maximum de 9 joueurs 
 */
void CRound::chargerNiveau(ifstream * fichier)
{
	char c;
	int x = 0;
	int y = 0;
	int numeroJoueurLu;	
	
	for ( c=fichier->get(); ! fichier->eof(); c=fichier->get())
	{	
		if (c == '\r')
			c=fichier->get();

		if ( c == '\n' )
		{	 
			x = 0;
			++y;	
		}
		
		else
		{
			switch (c)
			{
			case 'X':
					_tabCases[x][y] = new CCase(MUR_INDESTRUCTIBLE,NULL);
				break;
			case 'o':
					_tabCases[x][y] = new CCase(MUR_DESTRUCTIBLE,NULL);
				break;
			case ' ':
					_tabCases[x][y] = new CCase(CASE_LIBRE,NULL);
				break;
			default:
					numeroJoueurLu = (int)c - 48;
					if (  numeroJoueurLu < 0 || numeroJoueurLu > 9)
						assert (false);
					chargerJoueur(numeroJoueurLu, x, y);
				break;
			}
			x++;
		}
	}
}


void CRound::chargerJoueur(int numeroJoueurLu, int x, int y)
{
	if ( _nbJoueurs <= numeroJoueurLu )
		_tabCases[x][y] = new CCase(CASE_LIBRE,NULL);
	else
	{
		_tabCases[x][y] = new CCase(CASE_LIBRE, _vectAvatar[numeroJoueurLu]);
		_vectAvatar[numeroJoueurLu]->initialiserCaseDepart(x,y);
	}
}



CCase * CRound::getCase(int x, int y)
{
	assert( (x<NB_CASES_L) && (y<NB_CASES_H) );
	assert( (0<=x) && (0<=y) );
	return _tabCases[x][y];
}



void CRound::ajouterContenuADetruire(CContenu * ptADetruire)
{
	_vectADetruire.push_back(ptADetruire);
}




Uint32 oldTime;
Uint32 newTime;

int CRound::run()
{
	int dureePartie = DUREE_ROUND; //en secondes;
	bool bFinDePartie = false;
	int iNbJoueursVivants = _nbJoueurs;
	int iNumAvatarMort;
	int tiTailleTabDeTouches;		// on en a rien a faire mais necessaire pour SDL...
	Uint8 * ptClavier;

	vector<CContenu*>::iterator itDestruction;
	int y, x; // variable de parcourt de boucle


	// Variables de detection de fin de partie
	bool tabJoueursVivants[NB_JOUEURS_MAX];
	for (int i=0; i  < _nbJoueurs; i++)
		tabJoueursVivants[i]=true;
		
	// Variables de detection d'appui sur 'echap'
	

	// Pour l'affichage
	SDL_Rect coord_info;
	SDL_Rect posImage;	

	//prparation de l'affichage des scores
	SDL_Color jaune = {254,252,50};
	TTF_Font* police = TTF_OpenFont("resources/arbli___.ttf",20);
	assert(police != NULL);
	SDL_Surface* joueurs[4]; 
	
	joueurs[0]	= TTF_RenderText_Blended(police,"Joueur 1: ",jaune);
	joueurs[1]	= TTF_RenderText_Blended(police,"Joueur 2: ",jaune);
	joueurs[2]	= TTF_RenderText_Blended(police,"Joueur 3: ",jaune);
	joueurs[3]	= TTF_RenderText_Blended(police,"Joueur 4: ",jaune);


	SDL_Rect pos_joueurs[4];
	pos_joueurs[0].x = 5;
	pos_joueurs[0].y = 5;

	pos_joueurs[1].x = 5;
	pos_joueurs[1].y = 30;

	pos_joueurs[2].x = 200;
	pos_joueurs[2].y = 5;

	pos_joueurs[3].x = 200;
	pos_joueurs[3].y = 30;

	SDL_Surface* deuxPoints = TTF_RenderText_Blended(police,":",jaune) ;
	SDL_Surface* chiffres[10];
	chiffres[0] = TTF_RenderText_Blended(police,"0",jaune) ;
	chiffres[1] = TTF_RenderText_Blended(police,"1",jaune) ;
	chiffres[2] = TTF_RenderText_Blended(police,"2",jaune) ;	  
	chiffres[3] = TTF_RenderText_Blended(police,"3",jaune) ;	 
	chiffres[4] = TTF_RenderText_Blended(police,"4",jaune) ;
	chiffres[5] = TTF_RenderText_Blended(police,"5",jaune) ;
	chiffres[6] = TTF_RenderText_Blended(police,"6",jaune) ;
	chiffres[7] = TTF_RenderText_Blended(police,"7",jaune) ;
	chiffres[8] = TTF_RenderText_Blended(police,"8",jaune) ;
	chiffres[9] = TTF_RenderText_Blended(police,"9",jaune) ;


	Uint32 tpsdebut = SDL_GetTicks();
	_maMusique.jouer();

	
	while ( ! bFinDePartie )
	{
		// On teste si qq1 a appuye sur "echapp"
		SDL_PumpEvents();
		ptClavier = SDL_GetKeyState(&tiTailleTabDeTouches);
		if ( ptClavier[SDLK_ESCAPE] == 1 )
			exit(0);


		// Chaque case se gere et donc ses contenus se gerent (vivent) aussi
		for (y=0; y<NB_CASES_H; y++)
			for (x=0; x<NB_CASES_L; x++)
			{
				_tabCases[x][y]->gereToi();
			}

		// Reinitialisation de chacun des contenus (operation necessaire pour corriger le bug 
		// de conception qui faisait accelerer l'avatar qui allait vers la droite ou le bas
		for (y=0; y<NB_CASES_H; y++)
			for (x=0; x<NB_CASES_L; x++)
			{
				_tabCases[x][y]->reinitialiserVieDeContenu();
			}
		
		
		// Destruction des objet n'interragissant plus sur le plateau de jeu (ou alors
		//  on continue leur animation de fin de vie jusqu'a leur mort reelle
		for ( itDestruction=_vectADetruire.begin(); itDestruction != _vectADetruire.end(); itDestruction++)
		{
			if ( (*itDestruction)->estDetruit() )
			{
				delete (*itDestruction);
				_vectADetruire.erase(itDestruction);
				itDestruction--;
			}
			else
			{
				//continue destruction renvoie le numero d'avatar mort pr un avatar,
				//-1 pr le reste.		
				iNumAvatarMort = (*itDestruction)->continueDestruction();
				if (iNumAvatarMort >= 0 && tabJoueursVivants[iNumAvatarMort] != false)
				{
					tabJoueursVivants[iNumAvatarMort] = false;
					iNbJoueursVivants--;
				}
			}
		}

// AFFICHAGE
		posImage.x = 137;
		posImage.y = 109;	
		posImage.h = 659 - 109;
		posImage.w = 887 - 137;

		SDL_Rect posEcran;
		posEcran.x = 137;
		posEcran.y = 109;
		SDL_BlitSurface(_imageFond,&posImage,_ecran,&posEcran);

		for (y=0; y<NB_CASES_H; y++)
			for (x=0; x<NB_CASES_L; x++)
			{
				coord_info.x = TAILLE_SPRITE_X * x + DECALAGE_X;
				coord_info.y = TAILLE_SPRITE_Y * y + DECALAGE_Y;
				_tabCases[x][y]->afficheToi(coord_info, _ecran);	// chaque case s'affiche (ainsi
			}													// que son contenu) sur l'ecran
		
		
		SDL_BlitSurface(_imageDessus, NULL, _ecran, NULL);	
		
		//affichage des scores
		for (int i=0; i< _nbJoueurs; i++)
		{
			//affichage des joueurs
			SDL_BlitSurface(joueurs[i],NULL,_ecran,&pos_joueurs[i]);
			//affichage du nombre de rounds gagns.
			SDL_Rect pos_chiffre;
			pos_chiffre.x = pos_joueurs[i].x + 120;
			pos_chiffre.y = pos_joueurs[i].y;
			SDL_BlitSurface(chiffres[_partie[i]],NULL,_ecran,&pos_chiffre);
		}
		

//affichage du temps...
		Uint32 tpsCourant = SDL_GetTicks() - tpsdebut;
		
		//on dcrmente le temps restant si une seconde s'est coul.
		if (tpsCourant > 1000)
		{
			dureePartie --;
			tpsdebut = SDL_GetTicks();
		}

		int minutes = dureePartie / 60;
		int secondes = dureePartie - (minutes * 60);

	//afficher les minutes
		SDL_Rect pos_chiffre;
		pos_chiffre.x = 840;
		pos_chiffre.y = 30;
		SDL_BlitSurface(chiffres[minutes], NULL, _ecran, &pos_chiffre);
		pos_chiffre.x += 15;
		SDL_BlitSurface(deuxPoints, NULL, _ecran, &pos_chiffre);

	//afficher les secondes
		int dizaines = secondes / 10;
		int unites = secondes - (dizaines*10);

		pos_chiffre.x += 10;
		SDL_BlitSurface(chiffres[dizaines], NULL, _ecran, &pos_chiffre);
		pos_chiffre.x += 14;
		SDL_BlitSurface(chiffres[unites], NULL, _ecran, &pos_chiffre);

		
	//fin de l'affichage du temps...
	SDL_Flip(_ecran);
	
	// Calcul & affichage des FPS	
	oldTime = newTime;
	newTime = SDL_GetTicks();
	Uint32 delay = newTime - oldTime;
	float nbFPS = 1000.0 / (float)delay;
	cout << nbFPS << endl;
	
		// Detection de fin de partie (si la partie
		// est finie: 1 ou 0 joueurs restant en vie.
		if (iNbJoueursVivants == 0) 
		{
			bFinDePartie = true;
			_numVainqueur = 0;
		}

		else if (iNbJoueursVivants == 1) 
		{
			bFinDePartie = true;
			for (int i=0; i<_nbJoueurs; i++)
			{
				if ( tabJoueursVivants[i] == true )
					_numVainqueur = i + 1;
			}
		}

		//si le temps est ecoule, on arrete la partie
		else if (dureePartie == 0)
		{
			bFinDePartie = true;
			_numVainqueur = 0;
		}


	}
	

//fondu sur noir...
//BUG : fonctionne sur Linux en fenetre et plein ecran
//mais son windows en fenetre uniquement ! en plein ecran, il se
//passe un clignotement bien involontaire!!
// update 14/01/03: dut aux parametre  avancs des proprits
// d'affichage du panneau de configuration (il faut diminuer lacceleration materielle)
	Uint32 dureeEcran = SDL_GetTicks() + 2000;
	Uint32 next = SDL_GetTicks() + 25;
	Uint32 actuel = SDL_GetTicks();
	int alpha = 0;
		
	SDL_Surface* fondEcran = ImageLoad("images/black.bmp");
	
	bool _valider = false;
	while ( ! _valider )
	{
		
		actuel = SDL_GetTicks();

		if ( actuel > dureeEcran)
			_valider = true;

		else if (actuel > next)
		{	
			alpha += 1;
			SDL_SetAlpha(fondEcran, SDL_SRCALPHA, alpha);
			next += 75;
		}

		
		SDL_BlitSurface(fondEcran, NULL, _ecran, NULL);	
		SDL_Flip(_ecran);
	}
	
	SDL_FreeSurface(fondEcran);
	for (int j=0;j<10;j++)
	{
		SDL_FreeSurface(chiffres[j]);
		if (j < 4)
			SDL_FreeSurface(joueurs[j]);
	}

	_maMusique.stop();
	return _numVainqueur;
}




